@@ -20,6 +20,20 @@ This private submodule is *not* intended for importation by downstream callers.
 '''
 
 # ....................{ TODO                              }....................
+#FIXME: Refactor *ALL* calls to str.replace() throughout the codebase to call a
+#newly defined beartype._util.text.utiltextmunge.replace_str_substrs() function
+#whose signature should resemble:
+#    def replace_str_substrs(text: str, substr: str) -> str:
+#
+#The body of this function should:
+#* If the passed string does *NOT* contain at least one instance of the passed
+#  substring, raise an exception.
+#* Else, return:
+#      str.replace(text, substr)
+#
+#Why? Because the builtin str.replace() method performs *NO* such validation,
+#inviting non-human-readable exceptions when we inevitably muck things up.
+
 #FIXME: Major optimization: duplicate the signature of the decorated callable
 #as the signature of our wrapper function. Why? Because doing so obviates the
 #need to explicitly test whether each possible parameter was passed and how
@@ -123,23 +137,23 @@ This private submodule is *not* intended for importation by downstream callers.
 
 # ....................{ IMPORTS                           }....................
 from beartype.roar import BeartypeDecorParamNameException
-from beartype._decor._code._codesnip import (
+from beartype._decor._code.codesnip import (
     CODE_INIT_PARAMS_POSITIONAL_LEN,
     CODE_INIT_RANDOM_INT,
     CODE_RETURN_UNCHECKED,
     CODE_SIGNATURE,
-
-    # Attributes imported purely for public usage outside this subpackage.
-    PARAM_NAME_FUNC,
-    PARAM_NAME_TYPISTRY,
 )
-from beartype._decor._code._nonpep.nonpepcode import (
-    nonpep_code_check_param, nonpep_code_check_return)
 from beartype._decor._code._pep.pepcode import (
-    pep_code_check_param, pep_code_check_return)
+    coerce_hint_pep,
+    pep_code_check_param,
+    pep_code_check_return,
+)
 from beartype._decor._data import BeartypeData
-from beartype._util.hint.utilhinttest import die_unless_hint, is_hint_ignorable
-from beartype._util.hint.pep.utilhintpeptest import is_hint_pep
+from beartype._util.hint.utilhinttest import is_hint_ignorable
+from beartype._util.text.utiltextlabel import (
+    label_callable_decorated_param,
+    label_callable_decorated_return,
+)
 from inspect import Parameter, Signature
 
 # See the "beartype.__init__" submodule for further commentary.
@@ -195,9 +209,6 @@ def generate_code(data: BeartypeData) -> None:
     data : BeartypeData
         Decorated callable to be type-checked.
 
-    .. _PEP 484:
-        https://www.python.org/dev/peps/pep-0484
-
     Returns
     ----------
     (str, bool)
@@ -253,6 +264,9 @@ def generate_code(data: BeartypeData) -> None:
         If any type hint annotating any parameter of this callable is
         **unhashable** (i.e., *not* hashable by the builtin :func:`hash`
         function and thus unusable in hash-based containers like dictionaries).
+
+    .. _PEP 484:
+        https://www.python.org/dev/peps/pep-0484
     '''
     assert data.__class__ is BeartypeData, f'{repr(data)} not @beartype data.'
 
@@ -334,23 +348,31 @@ def _code_check_params(data: BeartypeData) -> 'Tuple[str, bool]':
     BeartypeDecorHintNonPepException
         If any type hint annotating any parameter of this callable is neither:
 
-        * **PEP-compliant** (i.e., :mod:`beartype`-agnostic hint compliant with
-          annotation-centric PEPs).
-        * **PEP-noncompliant** (i.e., :mod:`beartype`-specific type hint *not*
-          compliant with annotation-centric PEPs)).
-    TypeError
-        If any type hint annotating any parameter of this callable is
-        **unhashable** (i.e., *not* hashable by the builtin :func:`hash`
-        function and thus unusable in hash-based containers like dictionaries).
+        * A PEP-noncompliant type hint.
+        * A supported PEP-compliant type hint.
     '''
     assert data.__class__ is BeartypeData, f'{repr(data)} not @beartype data.'
 
+    # Decorated callable.
+    func = data.func
+
     # Python code snippet to be returned.
     func_code = ''
 
     # Python code snippet type-checking the current parameter.
     func_code_param = ''
 
+    # Name and kind of the current parameter.
+    param_name = None
+    param_kind = None
+
+    # Human-readable label describing the current parameter.
+    pith_label = None
+
+    # Type hint annotating this parameter if any *OR* "_PARAM_HINT_EMPTY"
+    # otherwise (i.e., if this parameter is unannotated).
+    hint = None
+
     # True only if type-checking for these parameters requires a higher-level
     # caller to prefix the body of this wrapper function with code generating
     # and localizing a pseudo-random integer.
@@ -367,77 +389,85 @@ def _code_check_params(data: BeartypeData) -> 'Tuple[str, bool]':
     # For the name of each parameter accepted by this callable and the
     # "Parameter" instance encapsulating this parameter (in declaration
     # order)...
-    for func_arg_index, func_arg in enumerate(
+    for param_index, param in enumerate(
         data.func_sig.parameters.values()):
+        # Type hint annotating this parameter if any *OR* "_PARAM_HINT_EMPTY"
+        # otherwise (i.e., if this parameter is unannotated).
+        hint = param.annotation
+
         # If this parameter is unannotated, continue to the next parameter.
-        if func_arg.annotation is _PARAM_HINT_EMPTY:
+        if hint is _PARAM_HINT_EMPTY:
             continue
         # Else, this parameter is annotated.
 
-        # If this parameter's type hint is unsupported, raise an exception.
-        die_unless_hint(func_arg.annotation)
-        # Else, this hint is guaranteed to be supported and thus hashable.
+        # Name and kind of the current parameter.
+        param_name = param.name
+        param_kind = param.kind
 
-        # If this callable redefines a parameter initialized to a default value
-        # by this wrapper, raise an exception. Permitting this unlikely edge
-        # case would permit unsuspecting users to accidentally override these
-        # defaults.
-        if func_arg.name.startswith('__bear'):
+        # Human-readable labels describing the current parameter and type
+        # hint annotating this parameter.
+        pith_label = label_callable_decorated_param(func, param_name)
+
+        # If this parameter's name is reserved for use by the @beartype
+        # decorator, raise an exception.
+        if param_name.startswith('__bear'):
             raise BeartypeDecorParamNameException(
-                f'{data.func_name} parameter '
-                f'"{func_arg.name}" reserved by @beartype.'
-            )
-
-        # If either this hint *OR* the type of this parameter is silently
-        # ignorable, continue to the next parameter.
-        #
-        # Note the "in" operator when applied to a set() object requires that
-        # the first operand (i.e., "func_arg_hint" and "func_arg_kind") to be
-        # hashable. Since:
-        #
-        # * "func_arg.annotation" is guaranteed to be hashable by the prior
-        #   call to the die_unless_hint() validator.
-        # * "func_arg_kind" derives from the stdlib "inspect" module, this
-        #   should *ALWAYS* be the case for that object.
-        if (
-            is_hint_ignorable(func_arg.annotation) or
-            func_arg.kind in _PARAM_KINDS_IGNORABLE
-        ):
+                f'{pith_label} reserved by @beartype.')
+        # If either the type of this parameter is silently ignorable, continue
+        # to the next parameter.
+        elif param_kind in _PARAM_KINDS_IGNORABLE:
             continue
         # Else, this parameter is non-ignorable.
+
+        # PEP-compliant type hint converted from this PEP-noncompliant type
+        # hint if this hint is PEP-noncompliant, this hint as is if this hint
+        # is both PEP-compliant and supported, *OR* raise an exception
+        # otherwise (i.e., if this hint is neither PEP-noncompliant nor a
+        # supported PEP-compliant hint).
+        hint = coerce_hint_pep(
+            func=func,
+            pith_name=param_name,
+            hint=hint,
+            hint_label=f'{pith_label} type hint',
+        )
+
+        # If this hint is ignorable, continue to the next parameter.
         #
-        # If this non-ignorable parameter either may or must be passed
-        # positionally, note this fact.
-        elif func_arg.kind is Parameter.POSITIONAL_OR_KEYWORD:
+        # Note that this is intentionally tested *AFTER* this hint has been
+        # coerced into a PEP-compliant type hint to implicitly ignore
+        # PEP-noncompliant type hints as well (e.g., "(object, int, str)").
+        if is_hint_ignorable(hint):
+            # print(f'Ignoring {data.func_name} parameter {param_name} hint {repr(hint)}...')
+            continue
+        # Else, this hint is unignorable.
+        #
+        # If this unignorable parameter either may *OR* must be passed
+        # positionally, record this fact. Note this conditional branch must be
+        # tested after validating this parameter to be unignorable; if this
+        # branch were instead nested *BEFORE* validating this parameter to be
+        # unignorable, @beartype would fail to reduce to a noop for otherwise
+        # ignorable callables -- which would be rather bad, really.
+        elif param_kind is Parameter.POSITIONAL_OR_KEYWORD:
             is_params_positional = True
 
-        # If this is a PEP-compliant hint...
-        if is_hint_pep(func_arg.annotation):
-            # Python code snippet type-checking the current parameter.
-            func_code_param, is_func_code_param_needs_random_int = (
-                pep_code_check_param(
-                    data=data,
-                    func_arg=func_arg,
-                    func_arg_index=func_arg_index,
-                ))
-
-            # Append code type-checking this parameter against this hint.
-            func_code += func_code_param
-
-            # If type-checking this parameter requires first localizing a
-            # pseudo-random integer, note that.
-            is_func_code_needs_random_int = (
-                is_func_code_needs_random_int or
-                is_func_code_param_needs_random_int
-            )
-        # Else, this is a PEP-noncompliant hint. In this case, append Python
-        # code type-checking this parameter against this hint.
-        else:
-            func_code += nonpep_code_check_param(
+        # Python code snippet type-checking this parameter against this hint.
+        func_code_param, is_func_code_param_needs_random_int = (
+            pep_code_check_param(
                 data=data,
-                func_arg=func_arg,
-                func_arg_index=func_arg_index,
-            )
+                hint=hint,
+                param=param,
+                param_index=param_index,
+            ))
+
+        # Append code type-checking this parameter against this hint.
+        func_code += func_code_param
+
+        # If type-checking this parameter requires first localizing a
+        # pseudo-random integer, note that.
+        is_func_code_needs_random_int = (
+            is_func_code_needs_random_int or
+            is_func_code_param_needs_random_int
+        )
 
     # Return all metadata required by higher-level callers, including...
     return (
@@ -497,6 +527,9 @@ def _code_check_return(data: BeartypeData) -> 'Tuple[str, bool]':
     '''
     assert data.__class__ is BeartypeData, f'{repr(data)} not @beartype data.'
 
+    # Decorated callable.
+    func = data.func
+
     # Python code snippet to be returned.
     func_code = None
 
@@ -505,35 +538,39 @@ def _code_check_return(data: BeartypeData) -> 'Tuple[str, bool]':
     # and localizing a pseudo-random integer.
     is_func_code_needs_random_int = False
 
-    # Type hint annotating this callable's return value if any *OR*
-    # "Signature.Empty" otherwise.
-    func_return_hint = data.func_sig.return_annotation
+    # Type hint annotating this callable's return if any *OR*
+    # "_RETURN_HINT_EMPTY" otherwise (i.e., if this return is unannotated).
+    hint = data.func_sig.return_annotation
 
-    # If this return value is unannotated, generate code calling this callable
+    # If this return is unannotated, generate code calling this callable
     # unchecked and returning this value from this wrapper.
-    if func_return_hint is _RETURN_HINT_EMPTY:
+    if hint is _RETURN_HINT_EMPTY:
         func_code = CODE_RETURN_UNCHECKED
-    # Else, this return value is annotated.
+    # Else, this return is annotated.
     else:
-        # If this hint is unsupported, raise an exception.
-        die_unless_hint(func_return_hint)
-        # Else, this hint is guaranteed to be supported and thus hashable.
+        # PEP-compliant type hint converted from this PEP-noncompliant type
+        # hint if this hint is PEP-noncompliant, this hint as is if this hint
+        # is both PEP-compliant and supported, *OR* raise an exception
+        # otherwise (i.e., if this hint is neither PEP-noncompliant nor a
+        # supported PEP-compliant hint).
+        hint = coerce_hint_pep(
+            func=func,
+            pith_name='return',
+            hint=hint,
+            hint_label=(
+                f'{label_callable_decorated_return(func)} type hint'),
+        )
 
-        # If this hint is silently ignorable, generate code calling this
-        # callable unchecked and returning this value from this wrapper.
-        if is_hint_ignorable(func_return_hint):
+        # If this hint is ignorable, generate code calling this callable
+        # unchecked and returning that return value from this wrapper.
+        if is_hint_ignorable(hint):
+            # print(f'Ignoring {data.func_name} return hint {repr(hint)}...')
             func_code = CODE_RETURN_UNCHECKED
-        # Else, this hint is *NOT* ignorable.
-        #
-        # If this is a PEP-compliant hint, generate Python code type-checking
-        # this return value against this hint.
-        elif is_hint_pep(func_return_hint):
-            func_code, is_func_code_needs_random_int = (
-                pep_code_check_return(data))
-        # Else, this is a PEP-noncompliant hint. In this case, generate Python
-        # code type-checking this return value against this hint.
+        # Else, this hint is unignorable.
         else:
-            func_code = nonpep_code_check_return(data)
+            # Python code snippet type-checking this return against this hint.
+            func_code, is_func_code_needs_random_int = pep_code_check_return(
+                data=data, hint=hint)
 
     # Return all metadata required by higher-level callers, including...
     return (
