@@ -43,14 +43,22 @@ This private submodule is *not* intended for importation by downstream callers.
 # ....................{ IMPORTS                           }....................
 from beartype.roar import (
     BeartypeDecorHintPepException,
+    BeartypeDecorHintPepDeprecatedWarning,
     BeartypeDecorHintPepUnsupportedException,
     # BeartypeDecorHintPepIgnorableDeepWarning,
     # BeartypeDecorHintPepUnsupportedWarning,
 )
 from beartype._util.cache.utilcachecall import callable_cached
 from beartype._util.hint.data.pep.utilhintdatapep import (
+    HINT_PEP_SIGNS_DEPRECATED,
     HINT_PEP_SIGNS_SUPPORTED,
 )
+from beartype._util.hint.data.pep.proposal.utilhintdatapep484 import (
+    HINT_PEP484_SIGNS_TYPE_ORIGIN,
+    HINT_PEP484_TUPLE_EMPTY,
+)
+from beartype._util.hint.data.pep.proposal.utilhintdatapep585 import (
+    HINT_PEP585_TUPLE_EMPTY)
 from beartype._util.hint.pep.proposal.utilhintpep484 import (
     is_hint_pep484_generic,
     is_hint_pep484_ignorable_or_none,
@@ -59,15 +67,16 @@ from beartype._util.hint.pep.proposal.utilhintpep484 import (
 from beartype._util.hint.pep.proposal.utilhintpep544 import (
     is_hint_pep544_ignorable_or_none)
 from beartype._util.hint.pep.proposal.utilhintpep585 import (
-    is_hint_pep585)
+    is_hint_pep585,
+    is_hint_pep585_generic,
+)
 from beartype._util.hint.pep.proposal.utilhintpep593 import (
     is_hint_pep593_ignorable_or_none)
 from beartype._util.utilobject import get_object_class_unless_class
-from beartype._util.py.utilpymodule import (
-    get_object_module_name, get_object_class_module_name_or_none)
+from beartype._util.py.utilpymodule import get_object_module_name
 from beartype._util.py.utilpyversion import IS_PYTHON_AT_LEAST_3_7
 from typing import TypeVar
-# from warnings import warn
+from warnings import warn
 
 # See the "beartype.__init__" submodule for further commentary.
 __all__ = ['STAR_IMPORTS_CONSIDERED_HARMFUL']
@@ -124,7 +133,7 @@ def die_if_hint_pep(
     hint_label : Optional[str]
         Human-readable label prefixing this object's representation in the
         exception message raised by this function. Defaults to
-        ``"Annotation"``.
+        ``"Type hint"``.
     exception_cls : Optional[type]
         Type of the exception to be raised by this function. Defaults to
         :class:`BeartypeDecorHintPepException`.
@@ -169,7 +178,7 @@ def die_unless_hint_pep(
         Object to be validated.
     hint_label : Optional[str]
         Human-readable label prefixing this object's representation in the
-        exception message raised by this function. Defaults to 'Annotation'.
+        exception message raised by this function. Defaults to ``"Type hint"``.
 
     Raises
     ----------
@@ -180,7 +189,6 @@ def die_unless_hint_pep(
     # If this hint is *NOT* PEP-compliant, raise an exception.
     if not is_hint_pep(hint):
         assert isinstance(hint_label, str), f'{repr(hint_label)} not string.'
-
         raise BeartypeDecorHintPepException(
             f'{hint_label} {repr(hint)} not PEP-compliant '
             f'(e.g., not "typing" type).')
@@ -263,24 +271,22 @@ def die_if_hint_pep_unsupported(
     # Regardless of whether it is or isn't, we raise a similar exception. Ergo,
     # there's no benefit to validating that expectation here.
     raise BeartypeDecorHintPepUnsupportedException(
-        f'{hint_label} PEP type hint {repr(hint)} '
+        f'{hint_label} PEP type hint "{repr(hint)}" '
         f'currently unsupported by @beartype.'
     )
 
 
 def die_if_hint_pep_sign_unsupported(
     # Mandatory parameters.
-    hint: object,
+    hint_sign: object,
 
     # Optional parameters.
-    hint_label: str = 'Unsubscripted "typing" attribute',
+    hint_label: str = 'Annotation sign',
 ) -> None:
     '''
-    Raise an exception unless the passed object is a **PEP-compliant supported
-    unsubscripted typing attribute** (i.e., public attribute of the
-    :mod:`typing` module without arguments uniquely identifying a category of
-    PEP-compliant type hints currently supported by the
-    :func:`beartype.beartype` decorator).
+    Raise an exception unless the passed object is a **supported PEP-compliant
+    sign** (i.e., arbitrary object uniquely identifying PEP-compliant type
+    hints currently supported by the :func:`beartype.beartype` decorator).
 
     This validator is intentionally *not* memoized (e.g., by the
     :func:`callable_cached` decorator), as the implementation trivially reduces
@@ -305,14 +311,78 @@ def die_if_hint_pep_sign_unsupported(
 
     # If this hint is *NOT* a supported unsubscripted "typing" attribute, raise
     # an exception.
-    if not is_hint_pep_sign_supported(hint):
+    if not is_hint_pep_sign_supported(hint_sign):
         assert isinstance(hint_label, str), f'{repr(hint_label)} not string.'
         raise BeartypeDecorHintPepUnsupportedException(
-            f'{hint_label} PEP sign {repr(hint)} '
+            f'{hint_label} PEP sign {repr(hint_sign)} '
             f'currently unsupported by @beartype.'
         )
 
 # ....................{ WARNINGS                          }....................
+def warn_if_hint_pep_sign_deprecated(
+    # Mandatory parameters.
+    hint: object,
+    hint_sign: object,
+
+    # Optional parameters.
+    hint_label: str = 'Annotated',
+) -> bool:
+    '''
+    Emit a non-fatal warning only if the passed PEP-compliant sign uniquely
+    identifying the passed PEP-compliant type hint is **deprecated** (e.g., due
+    to this outdated PEP-compliant type hint having since been obsoleted by one
+    or more recent PEPs).
+
+    This validator is intentionally *not* memoized (e.g., by the
+    :func:`callable_cached` decorator), as the implementation trivially reduces
+    to an efficient one-liner.
+
+    Parameters
+    ----------
+    hint : object
+        PEP-compliant type hint to be validated.
+    hint_sign : object
+        Arbitrary object uniquely identifying this hint.
+    hint_label : Optional[str]
+        Human-readable label prefixing this object's representation in the
+        warning message emitted by this function. Defaults to ``"Annotated"``.
+
+    Warns
+    ----------
+    BeartypeDecorHintPepDeprecatedWarning
+        If this sign is deprecated.
+    '''
+
+    # If this sign is deprecated...
+    if hint_sign in HINT_PEP_SIGNS_DEPRECATED:
+        assert isinstance(hint_label, str), f'{repr(hint_label)} not string.'
+
+        # Warning message to be emitted.
+        warning_message = f'{hint_label} PEP type hint {repr(hint)} deprecated'
+
+        # If this sign uniquely identifies PEP 484-compliant type hints
+        # originating from origin types (e.g., "typing.List[int]"), this sign
+        # has been deprecated by the equivalent PEP 585-compliant sign (e.g.,
+        # "list[int]"). In this case, suffix this warning message with
+        # pragmatic suggestions for resolving this deprecation.
+        if hint_sign in HINT_PEP484_SIGNS_TYPE_ORIGIN:
+            warning_message += (
+                ' by PEP 585. To resolve this, globally replace this hint by '
+                'the equivalent PEP 585 type hint '
+                '(e.g., "typing.List[int]" by "list[int]"). See also:\n'
+                '    https://www.python.org/dev/peps/pep-0585'
+            )
+        # Else, this sign is of unknown deprecation. In this case, simply
+        # terminate this unterminated sentence fragment as is.
+        else:
+            warning_message += '.'
+
+        # Emit this warning.
+        # print(f'Emitting {hint_label} hint {repr(hint)} deprecation warning...')
+        warn(warning_message, BeartypeDecorHintPepDeprecatedWarning)
+    # Else, this sign is *NOT* deprecated. In this case, reduce to a noop.
+
+
 #FIXME: Unit test us up.
 #FIXME: Actually use us in place of die_if_hint_pep_unsupported().
 #FIXME: Actually, it's unclear whether we still require or desire this. See
@@ -444,11 +514,11 @@ def is_hint_pep(hint: object) -> bool:
         is_hint_pep_class_typing(hint_type) or
         # This hint is a PEP 585-compliant type hint.
         is_hint_pep585(hint) or
-        # This hint is a PEP 484-compliant generic. Although a small subset of
+        # This hint is a PEP-compliant generic. Although a small subset of
         # generics are directly defined by the "typing" module (e.g.,
-        # "typing.IO"), most generics are user-defined subclasses defined by
-        # user-defined modules residing elsewhere.
-        is_hint_pep484_generic(hint) or
+        # "typing.SupportsInt"), most generics are user-defined subclasses
+        # defined by user-defined modules residing elsewhere.
+        is_hint_pep_generic(hint) or
         # This hint is a forward reference type hint.
         #
         # Note this unconditionally matches *ALL* forward references, including
@@ -590,13 +660,12 @@ def is_hint_pep_supported(hint: object) -> bool:
     # Else, this hint is PEP-compliant.
 
     # Avoid circular import dependencies.
-    from beartype._util.hint.pep.utilhintpepget import (
-        get_hint_pep_sign)
+    from beartype._util.hint.pep.utilhintpepget import get_hint_pep_sign
 
-    # Unsubscripted "typing" attribute uniquely identifying this hint.
+    # Sign uniquely identifying this hint.
     hint_pep_sign = get_hint_pep_sign(hint)
 
-    # Return true only if this attribute is supported.
+    # Return true only if this sign is supported.
     return is_hint_pep_sign_supported(hint_pep_sign)
 
 
@@ -781,6 +850,121 @@ is_hint_pep_class_typing.__doc__ = '''
           module.
     '''
 
+# ....................{ TESTERS ~ subtype : generic       }....................
+def is_hint_pep_generic(hint: object) -> bool:
+    '''
+    ``True`` only if the passed object is a **generic** (i.e., class
+    superficially subclassing at least one non-class PEP-compliant object).
+
+    Specifically, this tester returns ``True`` only if this object is a class
+    that is either:
+
+    * A `PEP 585`_-compliant generic as tested by the lower-level
+      :func:`is_hint_pep585_generic` function.
+    * A `PEP 484`_-compliant generic as tested by the lower-level
+      :func:`is_hint_pep484_generic` function.
+
+    This tester is intentionally *not* memoized (e.g., by the
+    :func:`callable_cached` decorator), as the implementation trivially reduces
+    to an efficient one-liner.
+
+    Parameters
+    ----------
+    hint : object
+        Object to be inspected.
+
+    Returns
+    ----------
+    bool
+        ``True`` only if this object is a generic.
+
+    See Also
+    ----------
+    :func:`is_hint_pep_typevared`
+        Commentary on the relation between generics and parametrized hints.
+
+    .. _PEP 484:
+       https://www.python.org/dev/peps/pep-0484
+    .. _PEP 585:
+       https://www.python.org/dev/peps/pep-0585
+    '''
+
+    # Return true only if this hint is...
+    return (
+        # A class that is either...
+        isinstance(hint, type) and (
+            # A PEP 585-compliant generic *OR*...
+            is_hint_pep585_generic(hint) or
+            # A PEP 484-compliant generic.
+            is_hint_pep484_generic(hint)
+        )
+    )
+
+# ....................{ TESTERS ~ subtype : tuple         }....................
+def is_hint_pep_tuple_empty(hint: object) -> bool:
+    '''
+    ``True`` only if the passed object is a PEP-compliant **empty fixed-length
+    tuple hint** (i.e., PEP-compliant type hint constraining piths to be the
+    empty tuple).
+
+    This tester r
+
+    This tester is intentionally *not* memoized (e.g., by the
+    :func:`callable_cached` decorator), as this tester is only called under
+    fairly uncommon edge cases.
+
+    Motivation
+    ----------
+    Since type variables are not themselves types but rather placeholders
+    dynamically replaced with types by type checkers according to various
+    arcane heuristics, both type variables and types parametrized by type
+    variables warrant special-purpose handling.
+
+    Parameters
+    ----------
+    hint : object
+        Object to be inspected.
+
+    Returns
+    ----------
+    bool
+        ``True`` only if this object is a type variable.
+
+    .. _PEP 484:
+       https://www.python.org/dev/peps/pep-0484
+    '''
+
+    # Return true only if this hint resembles either the PEP 484- or
+    # 585-compliant fixed-length empty tuple type hint. Since there only exist
+    # two such hints *AND* comparison against these hints is mostly fast, this
+    # test is efficient in the general case.
+    #
+    # Note that this test may also be inefficiently performed by explicitly
+    # obtaining this hint's sign and then subjecting this hint to specific
+    # tests conditionally depending on which sign and thus PEP this hint
+    # complies with: e.g.,
+    #     # Return true only if this hint is either...
+    #     return true (
+    #         # A PEP 585-compliant "tuple"-based hint subscripted by no
+    #         # child hints *OR*...
+    #         (
+    #             hint_curr_sign is tuple and
+    #             hint_childs_len == 0
+    #         ) or
+    #         # A PEP 484-compliant "typing.Tuple"-based hint subscripted
+    #         # by exactly one child hint *AND* this child hint is the
+    #         # empty tuple,..
+    #         (
+    #             hint_curr_sign is Tuple and
+    #             hint_childs_len == 1 and
+    #             hint_childs[0] == ()
+    #         )
+    #     )
+    return (
+        hint == HINT_PEP585_TUPLE_EMPTY or
+        hint == HINT_PEP484_TUPLE_EMPTY
+    )
+
 # ....................{ TESTERS ~ subtype : typevar       }....................
 def is_hint_pep_typevar(hint: object) -> bool:
     '''
